/*
 * Arduino Workbench tekyinblack 2020 - www.tekyinblack.com
 */
/*
 * setupResistance - setup input for Resistance measurements
 * drawResistance - obtain and put resistance values to the screen
 * logicResistance - hardware based routine to draw value on screen
 * resetPins - sets up pins for use in this application
 * checkResistance - when requested, returns a value to be displayed by drawResistance
 * stringResistance - returns a string as an E24 resistance value based on a passed absolute resistance value
 * getResistance - successively checks a test resistance against standard resistors to resolve raw value
 */
 
int resistanceReading=0;

// array of pins in use corrseponding to resistances associated with each
int testPins[] = {3, 4, 5, 6, 7, 8, 9, 10};
int testPinCount = sizeof(testPins)/ sizeof(testPinCount);
int resistancePin = 61; // pin A7 on a mega

// array of test resistances
long testResistance[8] = {1000000, 300000, 100000, 30000, 10000, 3300, 1000, 300};


// e24 series resistances with low and high tolerence overlaps
int e24[24][3] = {{95,100,105},
{105,110,115},
{115,120,125},
{125,130,139},
{139,150,155},
{155,160,170},
{170,180,190},
{190,200,210},
{210,220,230},
{230,240,254},
{254,270,284},
{284,300,314},
{314,330,344},
{344,360,374},
{374,390,409},
{409,430,450},
{450,470,490},
{490,510,533},
{533,560,588},
{588,620,650},
{650,680,713},
{713,750,780},
{780,820,862},
{862,910,950}};

int e24Value;
char outString[5];

void setupResistance() {
  // setup routine to initialise resistance function
  resetPins();
}

void drawResistance(int itemNo) {
  // resistance meter function, unique display, either small or full screen
  struct touchInfo newTouch;
   String resistanceValue = checkResistance();
  if (workMode == MODE_NORMAL) {
      newTouch = logicResistance(workScreens[itemNo][ITEMX], workScreens[itemNo][ITEMY], 5, resistanceValue, workScreens[itemNo][PARAM_2]);

  } else if (workMode == MODE_FULLSCREEN || workMode == MODE_LARGE){
      newTouch = logicResistance(0,50, 18, resistanceValue, workScreens[itemNo][PARAM_2]);

  }
  registerTouch(itemNo, newTouch);
}

struct touchInfo logicResistance(int xval, int yval, int textSize, String value, int drawColour) {
  // draw analog value on screen
  char data[10];
  int newWidth = 5*6*textSize;   //  5 characters wide, each of 6 pixels wide times text multiplier
  int newHeight = 8*textSize ;   // 8 pixels wide times text multiplier
  tft.setTextSize(textSize);
  tft.setTextColor(drawColour, BLACK);
  tft.setCursor(xval, yval);
  tft.print(value);
  if (value.length() < 4) {
    tft.print(" ");
  }

  return logicTouch(xval, xval + newWidth, yval, yval + newHeight);
}

void resetPins() {
  //  reset all pins to high immpedance
  for (int i=0;i<(testPinCount-1);i++) {
    digitalWrite(testPins[i], LOW);
    delayMicroseconds(100); 
    pinMode(testPins[i],INPUT);
    delayMicroseconds(100); 
  }
}

String checkResistance() {
  long resistance;
  // activate the highest resistance range to check if resistance attached
  // expects highest value standard resistance attached to test pin 0
  pinMode(testPins[0], OUTPUT);
  digitalWrite(testPins[0], HIGH);

  
  resistanceReading=analogRead(resistancePin);
  // if resistance detected then measure
  if (resistanceReading <= 1022) {
    digitalWrite(testPins[0], LOW);
    delayMicroseconds(100); 
    pinMode(testPins[0], INPUT);
    delayMicroseconds(100); 
     resistance = getResistance(resistanceReading);
  }
  else resistance=99999999;

  // return translated resistance
  return stringResistance(resistance);
}

String stringResistance(long newValue) {
  /*
   * Returns a string for display based on a passed absolute resistance value, mapping it to the closest E24 value
   */
  int j;
  // if a very large value then out of range
  if (newValue>9999999) {
      return("OoR ");  
    } 

  j=0;
  // reduce number into same range as comparison table and record decade
  // set to work over 7 decades
  for (int i=1; i<7; i++) {
    if (newValue < 95) {
      newValue = newValue * 10;
      j--;
      }
    else if (newValue <= 950) {
      break;
      }
    else if (newValue > 950) {
      newValue = newValue / 10;
      j++;
      }
    }

  // compare resistance value with e24 table to determine nominal value
  for (int i=0; i<24; i++) {
    if (newValue > e24[i][0] && newValue <= e24[i][2]) {
        e24Value = e24[i][1];
        break;
        }
      }
      
  // format resistance displayed to use R, k or M qualifiers
  // sprintf(outString, "%3d", e24Value);
    switch (j) {
      case -1:
         e24Value = e24Value/10;
         sprintf(outString, "%3dR", e24Value);
         break;
      case 0:
         sprintf(outString, "%3dR", e24Value);
         break;
     case 1:
         sprintf(outString,"%dk%d", e24Value/100, (e24Value%100)/10);
         break;
     case 2:
         sprintf(outString,"%2dk", e24Value/10);
         break;
     case 3:
         sprintf(outString,"%2dk", e24Value);
         break;
     case 4:
         sprintf(outString,"%dM%d", e24Value/100, (e24Value%100)/10);
         break;  
     case 5:
         sprintf(outString,"%2dM", e24Value/10);
         break;  
     default:
         sprintf(outString,"%s", "OoR ");
         break;    
        }
    // return display resistance value
    return(outString); 
}

long getResistance(int newReading) {
  /*
   *  Returns a value for the test resistance based on successively selecting test resistors until a voltage 
   *  closest to 512 units is determined.
   */
     long resistance;
     int oldDifference, newDifference;
     long bestTestResistance;
     // use passed reading as seed
     oldDifference=abs(511-newReading);

   for (int i=0; i<(testPinCount-1); i++) { // start of main for loop

    // setup pins to select next test resistor

    pinMode(testPins[i], OUTPUT);
    delayMicroseconds(100); 
    digitalWrite(testPins[i],HIGH);
    delayMicroseconds(100); 

    // read relative input voltage
    resistanceReading=analogRead(resistancePin);

    resistance = (resistanceReading * testResistance[i]) / (1023-resistanceReading);
      
      
  // optimum resistor ratio is closest to 50% so find and save value
      newDifference=abs(511-resistanceReading);

        if (newDifference < oldDifference) {
                bestTestResistance=resistance;
                oldDifference=newDifference;
          }
          
    digitalWrite(testPins[i], LOW);
    delayMicroseconds(100); 
    pinMode(testPins[i], INPUT);
    delayMicroseconds(100); 
    }  // end of main for loop

  return bestTestResistance; 

}
